package ac.essex.ooechs.fractals.swing;

import ac.essex.ooechs.fractals.maths.Fractal;
import ac.essex.ooechs.fractals.colouring.Colourer;

import javax.swing.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import java.awt.event.*;

/**
 * <p/>
 * A Swing Component which allows us to view a fractal in a java GUI.
 * Various mouse interactions are supported allowing the user to explore
 * the fractal.
 * </p>
 * <p/>
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 22-Nov-2007
 * @version 1.0
 */
public class FractalPanel extends JPanel {

    protected Fractal fractal;
    protected BufferedImage image;
    protected Colourer colourer;

    private int startX, startY;
    private Rectangle dragRegion;

    private boolean renderRequired;

    public FractalPanel(final FractalListener l, final Fractal fractal, final Colourer colourer) {

        this.fractal = fractal;
        this.colourer = colourer;

        // repaints the jpanel if it changes size
        addComponentListener(new ComponentAdapter() {

            public void componentResized(ComponentEvent e) {
                image = null;
                repaint();
            }

        });

        // allows the user to click the mouse or drag a zoom region
        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                startX = e.getX();
                startY = e.getY();
                dragRegion = new Rectangle(startX, startY, 1, 1);
            }

            public void mouseReleased(MouseEvent e) {
                if (image == null || renderRequired) return;

                if (e.getX() == startX && e.getY() == startY) {
                    fractal.center(e.getX(), e.getY(), image);
                } else {
                    fractal.zoom(dragRegion, image);
                }

                l.fractalUpdated(fractal);
                dragRegion = null;
                renderAgain();
            }

        });

        // allows the user to drag a zoom region
        addMouseMotionListener(new MouseMotionAdapter() {

            public void mouseDragged(MouseEvent e) {
                int x1 = Math.min(startX, e.getX());
                int x2 = Math.max(startX, e.getX());
                int y1 = Math.min(startY, e.getY());
                int y2 = Math.max(startY, e.getY());
                int width = x2 - x1;
                int height = y2 - y1;
                dragRegion = new Rectangle(x1, y1, width, height);
                repaint();
            }

        });

        // allows the user to zoom in or out using the scroll wheel
        addMouseWheelListener(new MouseWheelListener() {

            public void mouseWheelMoved(MouseWheelEvent e) {
                if (!renderRequired) {
                    dragRegion = null;
                    double zoom;
                    if (e.getWheelRotation() > 0) {
                        zoom = 1.5;
                    } else {
                        zoom = 0.75;
                    }
                    int centerX = getWidth() / 2;
                    int centerY = getHeight() / 2;
                    fractal.zoom(centerX, centerY, image, zoom);
                    l.fractalUpdated(fractal);
                    renderAgain();
                }
            }

        });

    }

    public void renderAgain() {
        renderRequired = true;
        repaint();
    }

    boolean rendering = false;

    protected void paintComponent(Graphics g) {
        if (image == null) {
            image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
            renderRequired = true;
        }
        if (renderRequired) {
            fractal.render(image, colourer);
            renderRequired = false;
        }
        g.drawImage(image, 0, 0, null);
        if (dragRegion != null) {
            g.drawRect(dragRegion.x, dragRegion.y, dragRegion.width, dragRegion.height);
        }

    }

}
